#!/bin/sh
# the next line restarts using wish \
    exec vtk "$0" "$@"


# -----------------------
# Dependencies and path
# -----------------------
package provide myutility 1.0
package require  Itcl 3.2
package require  Iwidgets 4.0

package require  pxitclbaseimagecontrol 1.0



itcl::class myutility {


    # derives from pxitclbaseimagecontrol

    inherit pxitclbaseimagecontrol
    protected common thisparam    

    #-----------------------------------
    # construction and descruction
    #-----------------------------------
    constructor { par } {
	pxitclbaseimagecontrol::constructor $par
    } { InitializeControl }
    
    destructor { }

    #-----------------------------------
    # initialization methods
    #-----------------------------------
    # This must exist it creates the interface 
    public method InitializeControl { }
    public method Initialize { inpwidg }
      
    #-----------------------------------
    # interface creation methods
    #-----------------------------------
    protected method CreateImagesControl { name }
    protected method CreateComputeControl { name }

    # ----------------------------------
    # Add this control to a Menu Button
    # ----------------------------------
    public method AddToMenuButton { mb args }    

    # ----------------------------------
    # Get Image and Points from Main Application
    # ----------------------------------
    public method GetPointsFromLandmarkControl { }
    public method GrabImage { image control }
    # ----------------------------------
    # Computational Utility Stuff
    # ----------------------------------
    public method Compute { }
    public method StoreResult { image { name "" } { orientation 0 } }
}  

#-------------------------------------------------------------------------------------------
#  Initialize tool
# -------------------------------------------------------------------------------------------
itcl::body myutility::InitializeControl { } {

    set appname "My Tool v1.0"
    set aboutstring "(c)  2006"


    # All Parameters that need to be appear in the GUI need to be part of this array
    # ------------------------------------------------------------------------------
    set thisparam($this,enablesmoothing) 1
    set thisparam($this,initialdistance) 2.0

    # We need three image controls 
    set thisparam($this,inputgui)      0
    set thisparam($this,initialphigui) 0
    set thisparam($this,outputgui)     0
}

#-------------------------------------------------------------------------------------------
itcl::body myutility::GrabImage { image control } {

    # image is a pxitclimage of the control
    # control is the image gui

    set gui $control
    puts stderr "GUI = $gui"

    set img [ $parent GetDisplayedImage ]
    if { [ $img GetImageSize ] > 0 } {
	[ $gui GetObject ]  ShallowCopy $img
	$gui Update
    }
}
#-------------------------------------------------------------------------------------------
itcl::body myutility::GetPointsFromLandmarkControl { } {

    set landmarkcontrol [ [ $parent GetLandmarkControl ] GetLandmarkControl ]
	
    set tempc_lv [ vtkpxBaseCurve [ pxvtable::vnewobj ]] 
    $tempc_lv Copy  [ $landmarkcontrol GetCollection -1 ]
    $tempc_lv Compact
    set pts [ $tempc_lv GetPoints ]
	
	
    set points [ vtkPoints [ pxvtable::vnewobj ]]
    $points DeepCopy $pts
    $tempc_lv Delete
    return $points


}
itcl::body myutility::Compute { } {

    set smoothing $thisparam($this,enablesmoothing)
    set distance  $thisparam($this,initialdistance)

    # This opens the console
    pxtkconsole

    # This prints to the console
    pxtkprint "Parameters\n-----------------\n"

    pxtkprint "Use Smoothing : $smoothing\n"
    pxtkprint "Initial Dist  : $distance\n"

    set points [ $this GetPointsFromLandmarkControl ]
    if { $points != 0 } {
	set np [ $points GetNumberOfPoints ]
	pxtkprint "Number of Points : $np\n"
	for { set i 0 } { $i < $np } { incr i } {
	    pxtkprint "Point $i : [ $points GetPoint $i ]\n"
	}
	pxtkprint "\n"
    }

    if { $points !=0 } {
	$points Delete
    }
}


# image is a vtkImageData Here !!!!
# $this StoreResult [ $myfilter GetOutput ] somename someorientation
itcl::body myutility::StoreResult { image { name "" } { orientation 0 } } {
    
    set gui $thisparam($this,outputgui)     
    [ $gui GetObject ]  ShallowCopy $img
    [ $gui GetObject ]  configure -filename $name
    [ $gui GetObject ]  configure -orientation $orientation
    $gui Update
}

#-------------------------------------------------------------------------------------------
# 
# ------------------------------------------------------------------------------------------
itcl::body myutility::CreateImagesControl { base } {

    set names [ list "inputgui" "initialphigui" "outputgui" ]
    set titles [ list "Input Image" "Initial LevelSet" "Output" ]

    for { set i 0 } { $i < [ llength $names ] } { incr i } {
	
	set guiname [ lindex $names $i ]
	set title   [ lindex $titles $i ]

	set  thisparam($this,$guiname) [ [ pxitclimageGUI \#auto  ] GetThisPointer ]
	$thisparam($this,$guiname) configure -description $title


	# $base.$i is a new frame that gets created and the control put in it
	$thisparam($this,$guiname) Initialize $base.$i 
	pack $base.$i -side top -expand f -fill x

	set bbut [ $thisparam($this,$guiname) cget -browsebutton ]
	pack forget $bbut

	$thisparam($this,$guiname) AddFunction "$parent SetImageFromObject" "Display" "$this"

	if { $guiname != "Output" } {
	    $thisparam($this,$guiname) AddFunction "$this GrabImage" "Grab" "$thisparam($this,$guiname)"
	}

    }
}

itcl::body myutility::CreateComputeControl { base } {


    iwidgets::labeledframe $base.frame0 \
	-labelpos nw  \
	-labeltext "Parameters"
    pack $base.frame0 -fill both -expand f -pady 5
    
    set frame0 [ $base.frame0 childsite ]

    checkbutton   $frame0.c -variable [ itcl::scope thisparam($this,enablesmoothing) ] -text "Enable Smoothing"
    iwidgets::entryfield $frame0.e -labeltext "Initial Distance:" -textvariable [ itcl::scope thisparam($this,initialdistance) ] -relief sunken -width 6 -validate real
    pack $frame0.c $frame0.e -side left -padx 2 -fill x -expand false

    eval "button $base.but -text \"Compute\" -command { $this Compute }"
    pack $base.but -side bottom -expand t -fill x
       
}

# -------------------------------------------------------------------------------------------
#  Main control inititialization
# ------------------------------------------------------------------------------------------
itcl::body myutility::Initialize { widget } {
    
    if { $initialized == 1 } { return $basewidget }
    
    #  -------------------------------------
    #  Create User Interface
    #  -------------------------------------	
    set basewidget [toplevel $widget ]
    wm geometry $basewidget 610x450
    wm withdraw $basewidget

    set notebook $basewidget.notebook    
    iwidgets::tabnotebook $notebook -tabpos w         
    set widget_list(notebook) $notebook

    set mb [ frame $basewidget.mb ]
    pack $mb -side top -fill x -expand false
    
    #    CreateMenu $mb
    CreateImagesControl     [ $notebook add -label "Images"  ]
    CreateComputeControl    [ $notebook add -label "Compute" ] 
    pack $notebook -side top -fill both -expand t -padx 5

    set initialized 1

    SetTitle "My Utility"

    # this is critical 
    eval "wm protocol $basewidget WM_DELETE_WINDOW { wm withdraw $basewidget }"
    return $basewidget 
}

#-------------------------------------------------------------------------------------------
#  Add controls to menu button
# ------------------------------------------------------------------------------------------
itcl::body myutility::AddToMenuButton { mb args } {
    eval "$mb add command -label \"Images\" -command {$this ShowWindow \"Images\"}"
    eval "$mb add command -label \"Compute\" -command {$this ShowWindow \"Compute\"}"
}

#-------------------------------------------------------------------------------------------
#  Main function for stand-alone execution
# ------------------------------------------------------------------------------------------
if { [ file rootname $argv0 ] == [ file rootname [ info script ] ] } {

    puts "\n[ file rootname $argv0 ] is not a stand-alone program.\n"
    exit
}

