// This corresponds to script10-5.tcl

#include <vtkPoints.h>
#include <vtkCellArray.h>
#include <vtkPolyData.h>
#include <vtkMath.h>
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkPolyDataMapper.h>
#include <vtkProperty.h>
#include <vtkCamera.h>
#include <vtkRenderWindowInteractor.h>

int main(int argc,char *argv[])
{
  double pi=vtkMath::Pi();

  vtkPoints* pts=vtkPoints::New();
  pts->SetNumberOfPoints(9);
  for (int i=0;i<=7;i++)
    {
      double rad= 2.0*double(i)*pi/8.0;
      double x  = 10.0* sin(rad);
      double y  = 10.0* cos(rad);
      pts->SetPoint(i,x,y,0.0);
      double* p=pts->GetPoint(i);
      fprintf(stdout,"Point %d = (%.2f,%.2f,%.2f)\n",i,p[0],p[1],p[2]);
    }
  pts->SetPoint(8,0.0,0.0,10.0);

  vtkCellArray* triangles=vtkCellArray::New();
  triangles->Allocate(10, 5);
  int p[3];
  for (int i=0;i<=7;i++)
    {
      p[0]=i;
      p[1]=i+1;
      if ( p[1] > 7)
	p[1]=0;
      p[2]=8;
	  
      triangles->InsertNextCell(3,p);
      fprintf(stdout,"Set Triangle %d = (%d,%d,%d)\n",i,p[0],p[1],p[2]);
    }

  vtkPolyData* surface=vtkPolyData::New();
  surface->SetPoints(pts);
  surface->SetPolys(triangles);


  // Since VTK uses reference counting delete pts and triangles
  // as these are counted inside surface
  pts->Delete();
  triangles->Delete();

  // Save the curve to a file
  // Create the mapper
  vtkPolyDataMapper* map=vtkPolyDataMapper::New();
  map->SetInput(surface);

  //Create the actor and set it to display wireframes
  vtkActor* actor=vtkActor::New();
  actor->SetMapper(map);
  map->Delete();
  vtkProperty* property=actor->GetProperty();
  property->SetColor(1,1,1);
  property->SetAmbient(1.0);
  property->SetDiffuse(0.0);
  property->SetSpecular(0.0);
  property->SetRepresentationToWireframe();

  // Create the renderer
  vtkRenderer* ren= vtkRenderer::New();
  ren->AddActor(actor);

  // Set camera mode to Orthographic as opposed to Perspective
  ren->GetActiveCamera()->ParallelProjectionOn();

  // Rest is standard window/interactor etc.
  // Render Window 
  vtkRenderWindow* renWin=vtkRenderWindow::New();
  renWin->AddRenderer(ren);
  renWin->SetSize(300,300);

  // Interactor
  vtkRenderWindowInteractor* iren=vtkRenderWindowInteractor::New();
  iren->SetRenderWindow(renWin);
  iren->Initialize();
  iren->Start();

}
