#include "vtkImageExtractVOI.h"
#include "vtkObjectFactory.h"
#include "vtkImageData.h"
#include "vtkPointData.h"
#include "vtkImageReslice.h"

vtkStandardNewMacro(vtkImageExtractVOI);


// Construct object to extract all of the input data.
vtkImageExtractVOI::vtkImageExtractVOI()
{
  this->VOI[0] = this->VOI[2] = this->VOI[4] = 0;
  this->VOI[1] = this->VOI[3] = this->VOI[5] = 1;
  this->SampleRate[0] = this->SampleRate[1] = this->SampleRate[2] = 1;
}

// ---------------------------------------------------------------------------

void vtkImageExtractVOI::PrintSelf(ostream& os, vtkIndent indent)
{
  vtkSimpleImageToImageFilter::PrintSelf(os,indent);

  os << indent << "  VOI " << this->VOI[0] << ":" << this->VOI[1] << " ";
  os << indent << "  VOI " << this->VOI[2] << ":" << this->VOI[3] << " ";
  os << indent << "  VOI " << this->VOI[4] << ":" << this->VOI[5] << "\n";
  os << indent << "  SampleRate "<<this->SampleRate[0]<< ","<<this->SampleRate[1]<<","<<this->SampleRate[2]<<"\n";
}
// ---------------------------------------------------------------------------
void vtkImageExtractVOI::ExecuteInformation()
{
  int i, outDims[3], voi[6];
  int rate[3];
  int wholeExtent[6];

  this->vtkSimpleImageToImageFilter::ExecuteInformation();

  vtkImageData* input=this->GetInput();
  vtkImageData* output=this->GetOutput();
  
  if (this->GetInput() == NULL)
    {
      vtkErrorMacro("Missing input");
      return;
    }
  input->GetWholeExtent( wholeExtent );
  
  for ( i=0; i < 6; i++ )
    {
      voi[i] = this->VOI[i];
    }

  for ( i=0; i < 3; i++ )
    {
      if ( voi[2*i+1] > wholeExtent[2*i+1] )
	{
	  voi[2*i+1] = wholeExtent[2*i+1];
	}
      else if ( voi[2*i+1] < wholeExtent[2*i] )
	{
	  voi[2*i+1] = wholeExtent[2*i];
	}
      if ( voi[2*i] < wholeExtent[2*i] )
	{
	  voi[2*i] = wholeExtent[2*i];
	}
      else if ( voi[2*i] > wholeExtent[2*i+1] )
	{
	  voi[2*i] = wholeExtent[2*i+1];
	}
      
      if ( voi[2*i] > voi[2*i+1] )
	{
	  voi[2*i] = voi[2*i+1];
	}
      
      if ( (rate[i] = this->SampleRate[i]) < 1 )
	{
	  rate[i] = 1;
	}
      
      outDims[i] = (voi[2*i+1] - voi[2*i]) / rate[i] + 1;
      if ( outDims[i] < 1 )
	{
	  outDims[i] = 1;
	}
    }
  
  // This makes sense for sample rates of 1, 1, 1.
  wholeExtent[0] = 0;
  wholeExtent[1] = outDims[0] - 1;
  wholeExtent[2] = 0;
  wholeExtent[3] = outDims[1] - 1;
  wholeExtent[4] = 0;
  wholeExtent[5] = outDims[2] - 1;
  
  output->SetDimensions(outDims);
  output->SetWholeExtent( wholeExtent );

  double spacing[3]; input->GetSpacing(spacing);
  double origin[3];  input->GetOrigin(origin);

  output->SetSpacing(spacing[0]*rate[0],spacing[1]*rate[1],spacing[2]*rate[2]);
  output->SetOrigin(voi[0]*spacing[0]+origin[0],
		    voi[2]*spacing[1]+origin[1],
		    voi[4]*spacing[2]+origin[2]);




}
// ---------------------------------------------------------------------------
void vtkImageExtractVOI::SimpleExecute(vtkImageData* input, vtkImageData* output) 
{
  if (input==NULL)
    {
      vtkErrorMacro(<< "Bad Input");
      return;
    }

  int dim[3]; output->GetDimensions(dim);


  vtkImageReslice* resl=vtkImageReslice::New();
  resl->SetInput(input);
  resl->SetOutputOrigin(output->GetOrigin());
  resl->SetOutputSpacing(output->GetSpacing());
  resl->SetInterpolationMode(0);
  resl->SetOutputExtent(0,dim[0]-1,0,dim[1]-1,0,dim[2]-1);
  resl->OptimizationOff();
  resl->Update();

  output->ShallowCopy(resl->GetOutput());
  resl->Delete();
}



