#ifndef __vtkImageExtractVOI_h
#define __vtkImageExtractVOI_h

#include "vtkSimpleImageToImageFilter.h"

class vtkImageExtractVOI : public vtkSimpleImageToImageFilter
{
public:
  vtkTypeMacro(vtkImageExtractVOI,vtkSimpleImageToImageFilter);
  void PrintSelf(ostream& os, vtkIndent indent);
  
  // Description:
  // Construct object to extract all of the input data.
  static vtkImageExtractVOI *New();
  
  // Description:
  // Specify i-j-k (min,max) pairs to extract. The resulting structured points
  // dataset can be of any topological dimension (i.e., point, line, image, 
  // or volume). 
  vtkSetVector6Macro(VOI,int);
  vtkGetVectorMacro(VOI,int,6);

  // Description:
  // Set the sampling rate in the i, j, and k directions. If the rate is >
  // 1, then the resulting VOI will be subsampled representation of the
  // input.  For example, if the SampleRate=(2,2,2), every other point will
  // be selected, resulting in a volume 1/8th the original size.
  vtkSetVector3Macro(SampleRate, int);
  vtkGetVectorMacro(SampleRate, int, 3);

protected:

  vtkImageExtractVOI();
  ~vtkImageExtractVOI() {};
  vtkImageExtractVOI(const vtkImageExtractVOI&) {};
  void operator=(const vtkImageExtractVOI&) {};

  // Description:
  // Execute Stuff
  virtual void  ExecuteInformation();
  virtual void  SimpleExecute(vtkImageData* input, vtkImageData* output);

  // Description:
  // Data Members
  int VOI[6];
  int SampleRate[3];
};

#endif


