# -----------------------
# Dependencies and path
# -----------------------
package provide mrfutility 1.0
package require  Itcl 3.2
package require  Iwidgets 4.0
package require  pxitclbaseimagecontrol 1.0
package require loadmrfsegm

itcl::class mrfutility {


    # derives from pxitclbaseimagecontrol

    inherit pxitclbaseimagecontrol
    protected common thisparam    

    #-----------------------------------
    # construction and descruction
    #-----------------------------------
    constructor { par } {
	pxitclbaseimagecontrol::constructor $par
    } { InitializeControl }
    
    destructor { }

    #-----------------------------------
    # initialization methods
    #-----------------------------------
    # This must exist it creates the interface 
    public method InitializeControl { }
    public method Initialize { inpwidg }
      
    #-----------------------------------
    # interface creation methods
    #-----------------------------------
    protected method CreateOutputImageGUIControl { guiname name widget }
    protected method CreateMultiThresholdControl { name }
    protected method CreateKMeansControl { name }
    protected method CreateMRFControl    { name }

    # ----------------------------------
    # Add this control to a Menu Button
    # ----------------------------------
    public method AddToMenuButton { mb args }    

    # ----------------------------------
    # Computational Utility Stuff
    # ----------------------------------
    public method CheckImage {  image name operation verbosity }
    public method ComputeMultiThresholdSegmentation { }
    public method ComputeKMeansSegmentation { }
    public method ComputeMRFSegmentation { }


    public method ProcessResult { image guiname opname } 
}  

#-------------------------------------------------------------------------------------------
#  Initialize tool
# -------------------------------------------------------------------------------------------
itcl::body mrfutility::InitializeControl { } {

    set appname "Intensity Segmentation Tool v1.0"
    set aboutstring "(c) Xenios Papademetris, Nov 2006"


    # All Parameters that need to be appear in the GUI need to be part of this array
    # ------------------------------------------------------------------------------

    # Multi Threshold
    # ---------------------------------------------
    for { set i 0 } { $i < 5 } { incr i } {
	set thisparam($this,multi_thr$i) [ expr ($i+1) * 100.0 ]
	set thisparam($this,multi_on$i)  [ expr $i < 3 ]
    }

    # KMeans
    # ---------------------------------------------
    set thisparam($this,kmeans_numclasses) 3
    set thisparam($this,kmeans_iterations) 30
    set thisparam($this,kmeans_numbins)    128
    set thisparam($this,kmeans_convergence) 0.5
    set thisparam($this,kmeans_maxsigmaratio) 0.5


    # MRF
    # ---------------------------------------------
    set thisparam($this,mrf_numclasses) 3
    set thisparam($this,mrf_iterations) 8
    set thisparam($this,mrf_smoothness) 2.0
    set thisparam($this,mrf_noise)   5
    set thisparam($this,mrf_mrfiterations) 5
    set thisparam($this,mrf_convergence) 0.1
    set thisparam($this,mrf_initial) "K-Means"



    # We need three image controls to store outputs
    # ---------------------------------------------
    set thisparam($this,multithresholdoutput)      0
    set thisparam($this,kmeansoutput)              0
    set thisparam($this,mrfoutput)                 0

}

#-------------------------------------------------------------------------------------------
# image is a vtkImageData Here !!!!
# $this ProcessResult [ $myfilter GetOutput ] somename someorientation
itcl::body mrfutility::ProcessResult { image guiname opname } { 
    
    $currentresults ShallowCopyImage  $image
    $currentresults configure -filename  [ AddPrefix [ $currentimage cget -filename ] $opname ]
    $parent SetMaskFromObject $currentresults $this

    set gui $thisparam($this,$guiname)     
    [ $gui GetObject ]  ShallowCopy $currentresults
    $gui Update
}

#-------------------------------------------------------------------------------------------
#  Create GUI
# ------------------------------------------------------------------------------------------
# Algorithm GUIs
# -------------------------------------------------------------------------------------------
itcl::body mrfutility::CreateOutputImageGUIControl { guiname name widget } {

    set  thisparam($this,$guiname) [ [ pxitclimageGUI \#auto  ] GetThisPointer ]
    $thisparam($this,$guiname) configure -description $name
    $thisparam($this,$guiname) Initialize $widget
    set bbut [ $thisparam($this,$guiname) cget -browsebutton ];	pack forget $bbut
    $thisparam($this,$guiname) AddFunction "$parent SetResultsFromObject" "Display" "$this"
    $thisparam($this,$guiname) AddFunction "$parent SetMaskFromObject" "Display Mask" "$this"

    return $widget
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

itcl::body mrfutility::CreateMultiThresholdControl { base } {

    iwidgets::labeledframe $base.frame0 \
	-labelpos nw  \
	-labeltext "Multi Threshold Segmentation"
    pack $base.frame0 -fill both -expand f -pady 5
    
    set frame0 [ $base.frame0 childsite ]

    for { set l 0 } { $l < 5 } { incr l } {
	set w [ frame $frame0.$l ] ; pack $w -side top -expand f -fill x
	eval "iwidgets::entryfield  $w.0 -labeltext \"Threshold:\" -textvariable [ itcl::scope  thisparam($this,multi_thr$l) ] -width 6 -validate real"
	eval "checkbutton $w.c -variable [ itcl::scope thisparam($this,multi_on${l}) ] -text Use "
	pack $w.0 $w.c  -side left -expand f -padx 5
    }




    eval "button $base.but -text \"Compute Multi-Threshold Segmentation\" -command { $this ComputeMultiThresholdSegmentation }"
    pack $base.but -side top -expand t -fill x

    set widg [ $this CreateOutputImageGUIControl "multithresholdoutput" "Multi-Threshold Output" $base.bot ]
    pack $widg -side bottom -expand f -fill x
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

itcl::body mrfutility::CreateKMeansControl { base } {

    iwidgets::labeledframe $base.frame0 \
	-labelpos nw  \
	-labeltext "KMeans Segmentation"
    pack $base.frame0 -fill both -expand f -pady 5
    
    set w [ $base.frame0 childsite ]; set k 0

    iwidgets::entryfield  $w.$k -labeltext "Classes:" -textvariable [ itcl::scope  thisparam($this,kmeans_numclasses) ] -width 2 -validate integer
    pack $w.$k -side top -expand true -fill x; incr k
    
    iwidgets::entryfield  $w.$k -labeltext "Iterations:" -textvariable [ itcl::scope  thisparam($this,kmeans_iterations) ] -width 2 -validate integer
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "Bins:" -textvariable [ itcl::scope  thisparam($this,kmeans_numclasses) ] -width 3 -validate integer
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "Max Sigma Ratio (0.0-1.0):" -textvariable [ itcl::scope  thisparam($this,kmeans_maxsigmaratio) ] -width 3 -validate real
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "Convergence:" -textvariable [ itcl::scope  thisparam($this,kmeans_convergence) ] -width 4 -validate real
    pack $w.$k -side top -expand true -fill x; incr k

    
    iwidgets::Labeledwidget::alignlabels $w.0 $w.1 $w.2 $w.3 $w.4

    eval "button $base.but -text \"Compute K-Means Segmentation\" -command { $this ComputeKMeansSegmentation }"
    pack $base.but -side top -expand t -fill x

    set widg [ $this CreateOutputImageGUIControl "kmeansoutput" "K-Means Output" $base.bot ]
    pack $widg -side bottom -expand f -fill x
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

itcl::body mrfutility::CreateMRFControl { base } {

    iwidgets::labeledframe $base.frame0 \
	-labelpos nw  \
	-labeltext "MRF Intensity Segmentation"
    pack $base.frame0 -fill both -expand f -pady 5
    
    set w [ $base.frame0 childsite ]; set k 0

    set cw [ [ iwidgets::labeledwidget  $w.$k -labeltext "Initialization:"] childsite ];
    pack $w.$k; incr k
    tk_optionMenu $cw.optmenu [ itcl::scope thisparam($this,mrf_initial) ] "Multi-Threshold" "K-Means"
    pack $cw.optmenu -side right -expand false -fill x


    iwidgets::entryfield  $w.$k -labeltext "Classes:" -textvariable [ itcl::scope  thisparam($this,mrf_numclasses) ] -width 2 -validate integer
    pack $w.$k -side top -expand true -fill x; incr k
    
    iwidgets::entryfield  $w.$k -labeltext "Iterations:" -textvariable [ itcl::scope  thisparam($this,mrf_iterations) ] -width 2 -validate integer
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "Smoothness:" -textvariable [ itcl::scope  thisparam($this,mrf_smoothness) ] -width 4 -validate real
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "Image Noise (0.0-100.0):" -textvariable [ itcl::scope  thisparam($this,mrf_noise) ] -width 3 -validate real
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "Convergence:" -textvariable [ itcl::scope  thisparam($this,kmeans_convergence) ] -width 4 -validate real
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::entryfield  $w.$k -labeltext "MRF Iterations:" -textvariable [ itcl::scope  thisparam($this,mrf_mrfiterations) ] -width 2 -validate integer
    pack $w.$k -side top -expand true -fill x; incr k

    iwidgets::Labeledwidget::alignlabels $w.0 $w.1 $w.2 $w.3 $w.4 $w.5 $w.6
    
    eval "button $base.but -text \"Compute MRF Segmentation\" -command { $this ComputeMRFSegmentation }"
    pack $base.but -side top -expand t -fill x

    set widg [ $this CreateOutputImageGUIControl "mrfoutput" "MRF Output" $base.bot ]
    pack $widg -side bottom -expand f -fill x
}

# -------------------------------------------------------------------------------------------
#  Main control inititialization
# ------------------------------------------------------------------------------------------
itcl::body mrfutility::Initialize { widget } {
    
    if { $initialized == 1 } { return $basewidget }
    
    #  -------------------------------------
    #  Create User Interface
    #  -------------------------------------	
    set basewidget [toplevel $widget ]
    wm geometry $basewidget 610x450
    wm withdraw $basewidget

    set notebook $basewidget.notebook    
    iwidgets::tabnotebook $notebook -tabpos w         
    
    #    CreateMenu $mb
    CreateMultiThresholdControl    [ $notebook add -label "MultiThreshold" ] 
    CreateKMeansControl            [ $notebook add -label "KMeans" ] 
    CreateMRFControl               [ $notebook add -label "MRF" ] 
    pack $notebook -side top -fill both -expand t -padx 5

    set initialized 1

    SetTitle "Intensity Segmentation Tool"

    # this is critical 
    eval "wm protocol $basewidget WM_DELETE_WINDOW { wm withdraw $basewidget }"
    return $basewidget 
}
#-------------------------------------------------------------------------------------------
# Segmentation Code Here
#-------------------------------------------------------------------------------------------
itcl::body mrfutility::CheckImage {  image name operation verbosity } {
    
    set ok 0
    
    catch { if { [ $image GetImageSize ] > 1 } {  set ok 1 }   }
    
    if { $verbosity >0 && $ok == 0 } {
	::pxtclutil::Warning "No $name in Memory\n Cannot perform $operation !"
    }

    return $ok
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
itcl::body mrfutility::ComputeMultiThresholdSegmentation {  } {

    if { [ $this CheckImage $currentimage "Image" "Multi-Threshold Segmentation" 1 ] == 0 } {
	return 0
    }

    set map [ vtkDoubleArray [ pxvtable::vnewobj ]]
    $map SetNumberOfComponents 1

    for { set l 0 } { $l < 5 } { incr l } {
	puts stderr "Flag $l = $thisparam($this,multi_on$l)"
	if { $thisparam($this,multi_on$l) > 0 } {
	    set x $thisparam($this,multi_thr$l) 
	    $map InsertNextTuple1 $x
	}
    }

    if { [ $map  GetNumberOfTuples ] < 1 } {
	::pxtclutil::Warning "No Thresholds set!\n Cannot perform MultiThresholding.\n"
	return 0
    }
    
    WatchOn
    set segm [ vtkMultiThresholdSegmentation [ pxvtable::vnewobj ]]
    $segm SetInput [ $currentimage GetImage ]
    $segm SetThresholds $map
    $segm Update
    WatchOff

    $map Delete
    $this ProcessResult [ $segm GetOutput ] "multithresholdoutput" "mthr"
    $segm Delete
    
}
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
itcl::body mrfutility::ComputeKMeansSegmentation {  } {
    if { [ CheckImage $currentimage "Image" "K-Means Segmentation" 1 ] == 0 } {
	return 0
    }

    WatchOn
    set segm [ vtkKMeansSegmentation [ pxvtable::vnewobj ]]
    $segm SetInput [ $currentimage GetImage ]
    $segm SetNumberOfBins    $thisparam($this,kmeans_numbins)
    $segm SetNumberOfClasses $thisparam($this,kmeans_numclasses)
    $segm SetMaxSigmaRatio   $thisparam($this,kmeans_maxsigmaratio)    
    $segm SetIterations	     $thisparam($this,kmeans_iterations) 
    $segm SetConvergence     $thisparam($this,kmeans_convergence)
    $segm Update
    WatchOff

    $this ProcessResult [ $segm GetOutput ] "kmeansoutput" "km"
    $segm Delete

}
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
itcl::body mrfutility::ComputeMRFSegmentation {  } {
    if { [ CheckImage $currentimage "Image" "MRF Segmentation" 1 ] == 0 } {
	return 0
    }

    if { $thisparam($this,mrf_initial) == "K-Means" } {
	set initial [ $thisparam($this,kmeansoutput)  GetImage ]
    } else {
	set initial [ $thisparam($this,multithresholdoutput)  GetImage ]
    }

    if { [ CheckImage $initial "Initial Segmentation Image (from $thisparam($this,mrf_initial) segmentation output)" "MRF Segmentation" 1 ] == 0 } {
	return 0
    }

    WatchOn
    set segm [ vtkMRFSegmentation [ pxvtable::vnewobj ]]
    $segm SetInput [ $currentimage GetImage ]
    $segm SetInitialSegmentation [ $initial GetImage ]
    $segm SetNumberOfClasses       $thisparam($this,mrf_numclasses)
    $segm SetNumberOfIterations	   $thisparam($this,mrf_iterations) 
    $segm SetSmoothness            $thisparam($this,mrf_smoothness)    
    $segm SetImageNoisePercentage  $thisparam($this,mrf_noise)    
    $segm SetMRFIterations	   $thisparam($this,mrf_mrfiterations) 
    $segm SetConvergencePercentage $thisparam($this,mrf_convergence)
    $segm Update


    WatchOff

    $this ProcessResult [ $segm GetOutput ] "mrfoutput" "mrf"
    $segm Delete

}
#-------------------------------------------------------------------------------------------
#  Add controls to menu button
# ------------------------------------------------------------------------------------------
itcl::body mrfutility::AddToMenuButton { mb args } {
    eval "$mb add command -label \"Multi-Threshold\" -command {$this ShowWindow \"MultiThreshold\"}"
    eval "$mb add command -label \"K-Means\" -command {$this ShowWindow \"KMeans\"}"
    eval "$mb add command -label \"MRF-Segmentation\" -command {$this ShowWindow \"MRF\"}"
}

#-------------------------------------------------------------------------------------------
#  Main function for stand-alone execution
# ------------------------------------------------------------------------------------------
if { [ file rootname $argv0 ] == [ file rootname [ info script ] ] } {

    puts "\n[ file rootname $argv0 ] is not a stand-alone program.\n"
    exit
}

