#ifndef __vtkMRFSegmentation_h
#define __vtkMRFSegmentation_h

#include "vtkSimpleImageToImageFilter.h"
#include "vtkImageData.h"
#include "vtkDoubleArray.h"

class vtkMRFSegmentation : public vtkSimpleImageToImageFilter
{
public:
  static vtkMRFSegmentation *New();
  vtkTypeMacro(vtkMRFSegmentation,vtkSimpleImageToImageFilter);

  // Description:
  // Number Of Iterations (EM Iterations)
  vtkGetMacro(NumberOfIterations,int);
  vtkSetClampMacro(NumberOfIterations,int,0,20);

  // Description:
  // Number Of Iterations (MRF Smoothning Iterations)
  vtkGetMacro(MRFIterations,int);
  vtkSetClampMacro(MRFIterations,int,0,20);

  // Description:
  // Number Of NumberOfClasses
  vtkGetMacro(NumberOfClasses,int);
  vtkSetClampMacro(NumberOfClasses,int,0,20);

  // Description:
  // Weight of MRF Smoothness Term
  vtkGetMacro(Smoothness,double);
  vtkSetClampMacro(Smoothness,double,0,100.0);

  // Description:
  // Image Noise Term -- adds stability
  vtkGetMacro(ImageNoisePercentage,double);
  vtkSetClampMacro(ImageNoisePercentage,double,0,20.0);


  // Description:
  // Max Percentage of Voxels changing needed for iteration to converge (0 == all voxels)
  vtkGetMacro(ConvergencePercentage,double);
  vtkSetClampMacro(ConvergencePercentage,double,0,0.1);

  // Description:
  // Initial Segmentation Map as Image
  vtkSetObjectMacro(InitialSegmentation,vtkImageData);
  vtkGetObjectMacro(InitialSegmentation,vtkImageData);

  // Description:
  // Initial Segmentation Map as Image
  vtkGetObjectMacro(Parameters,vtkDoubleArray);


protected:

  vtkMRFSegmentation();
  virtual ~vtkMRFSegmentation();
  vtkMRFSegmentation(const vtkMRFSegmentation&) {};
  void operator=(const vtkMRFSegmentation&) {};


  // Description:
  // Basic Stuff
  virtual void ExecuteInformation();
  virtual void SimpleExecute(vtkImageData* in,vtkImageData* out);

  // Description:
			      
  // E-Step Is Easy for the most_part
  // Compute Parameter Estimates Given Classication
  // ----------------------------------------------
  virtual void DoExpectationStep(vtkImageData* intensities,vtkImageData* classification,vtkDoubleArray* params,int numclasses);

  // M-Step
  // -----------------------------------
  virtual double Metric(double x,double m,double sigma2);
  virtual double ComputeLogLikelihoodProbability(double intensity,int current_label,vtkDoubleArray* params);
  virtual double ComputeLogMRFPrior(vtkDataArray* labels,int vox,int current_label,int numclasses,int incr[6],double wgth[6]);
  virtual double ComputeTotalMinusLogProbability(double intensity,
						 int    current_voxel,
						 int    current_label,
						 vtkDataArray* labels,
						 vtkDoubleArray* parameters,
						 int numclasses,
						 int incr[6],double wgth[6],
						 double smoothness);
  
  virtual int  UpdateVoxel(int eveni,int evenj,int evenk,int pass);
  virtual int  ComputeMRFIncrementsAndWeights(vtkImageData* img,int incr[6],double wgt[6]);
  virtual int  ClassifyVoxel(int voxel_index,vtkDataArray* intensities,vtkDataArray* labels,vtkDoubleArray* params,
			     int numclasses,int incr[6],double wgth[6],
			     double smoothness);
  virtual double DoMaximizationStep(vtkImageData* intensity_image,
				    vtkImageData* label_image,
				    vtkDoubleArray* params,
				    int numclasses,
				    double smoothness,
				    int maxiter);



  // Descrtiption:
  // Key Variables
  double          Smoothness;
  double          ConvergencePercentage;
  double          Threshold;
  double          ImageNoiseVariance;
  double          ImageNoisePercentage;
  int             MRFIterations;
  int             NumberOfIterations;
  int             NumberOfClasses;
  vtkImageData*   InitialSegmentation;
  vtkDoubleArray* Parameters;

};

#endif
