#!/bin/sh
# the next line is executed by the shell, but it is a comment in tcl \
  exec wish "$0" "$@"

proc Reset { ask } {

    global parameters

    if { $ask == 1 } {
	set ok [  tk_messageBox -type yesno -default no -title "Think again ..." -message "Reseting filenames" -icon question  ]
	if { $ok == "no"  } {
	    return
	}
    }
    set parameters(readname)  ""
    set parameters(writename) ""
    set parameters(directory) ""
}

proc SetColor { widget  } {
    set color  [ tk_chooseColor -title "Set Background Color"  -parent . ]
    if { [ string length $color ] > 0 } {
	$widget configure -bg $color
    }
}

proc GetName { mode } {
    global parameters

    set fname ""
    set filetype1 [ list "Text Files" [ list .txt .tex ] ]
    set filetype2 [ list "All Files" "*"  ]

    switch -exact $mode {
	"readname" {
	    set fname [tk_getOpenFile -title "Load Filename" -filetypes [ list $filetype1 $filetype2 ] ]
	}
	"writename" {	  
	    set fname [tk_getSaveFile -title "Filename" -defaultextension .txt -initialfile default.txt  -filetypes [ list $filetype1 ] ]
	}
	"directory" {
	    set fname  [tk_chooseDirectory -title "Select Current Directory" ]
	}
    }

    if { [ string length $fname ] < 1 } {
	return
    }

    set parameters($mode) $fname
}

Reset 0

for { set i 1 } { $i <= 3 } { incr i } {
    
    set parent [ frame .frame$i ]
    pack $parent -side top -expand false -fill x

    switch -exact $i {
	"1" { 
	    set name "Input File:"
	    set mode readname
	}
	"2" {
	    set name "Save File :"
	    set mode writename
	}
	"3" {
	    set name "Directory :"
	    set mode directory
	}
    }
	
    label  $parent.1 -text $name -width 20
    entry  $parent.2 -textvariable parameters($mode) -width 80 
    eval "button $parent.3 -text \"...\" -command { GetName $mode }"
    pack $parent.3 -side right -expand false -padx 5
    pack $parent.1 -side left  -expand false -padx 2
    pack $parent.2 -side left -expand true -fill x
}

frame .bottom; pack .bottom -side bottom -expand true -fill x -pady 10 -padx 20

button .bottom.1 -text "Reset" -command { Reset 1 }
button .bottom.2 -text "Color" -command { SetColor . }
button .bottom.3 -text "Exit" -command { exit }
pack  .bottom.1 .bottom.2 .bottom.3 -side left -expand true -fill x -padx 2
